using System;
using System.IO;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Generic;

namespace MiniLib.Model
{
    public abstract partial class DocumentBase
    {
        private string _name;
        private bool _isDirty = true;

        public event EventHandler Changed;

        protected DocumentBase(string name)
        {
            Name = name;
        }

        public virtual string Name
        {
            get { return _name; }
            protected set
            {
                if (String.IsNullOrEmpty(value))
                    throw new ArgumentException("String.IsNullOrEmpty(value)");
                _name = value;
            }
        }

        public virtual bool IsDirty
        {
            get { return _isDirty; }
            protected set { _isDirty = value; }
        }

        public override string ToString()
        {
            return Name;
        }

        protected virtual void Invalidate()
        {
            IsDirty = true;
            OnChanged();
        }

        protected virtual void OnNewDocument()
        {
            IsDirty = true;
            OnChanged();
        }

        protected virtual void OnOpenDocument()
        {
            IsDirty = false;
            OnChanged();
        }

        protected virtual void OnSaveDocument()
        {
            IsDirty = false;
        }

        protected virtual void OnCloseDocument()
        {
        }

        protected virtual void OnChanged()
        {
            if (Changed != null)
                Changed(this, EventArgs.Empty);
        }
    }
}
