﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MiniLib.Persistence;

namespace MiniLib.Model
{
    public abstract partial class DocumentBase
    {
        public abstract class DocumentManagerBase
        {
            private DocumentBase _document = null;
            private readonly PersisterBase _persister;

            protected DocumentManagerBase(PersisterBase persister)
            {
                if (persister == null)
                    throw new ArgumentNullException("persister");
                _persister = persister;
            }

            public DocumentBase Document
            {
                get { return _document; }
                protected set { _document = value; }
            }

            public bool DocumentExists
            {
                get { return (Document != null); }
            }

            protected PersisterBase Persister
            {
                get { return _persister; }
            }

            //  Gestori dei servizi principali

            public virtual void NewDocument()
            {
                if (DocumentExists)
                    throw new ApplicationException("DocumentExists");
                Document = CreateDocument(DefaultName);
                Document.OnNewDocument();
            }

            public virtual bool OpeningDocument()
            {
                if (DocumentExists)
                    throw new ApplicationException("DocumentExists");
                return Persister.OpeningDocument();
            }

            public virtual void OpenDocument()
            {
                if (DocumentExists)
                    throw new ApplicationException("DocumentExists");
                Document = CreateDocument(Persister.LocalName);
                Persister.LoadDocument(Document);
                Document.OnOpenDocument();
            }

            public virtual bool SavingDocument(bool askForName)
            {
                if (!DocumentExists)
                    throw new ApplicationException("!DocumentExists");
                return Persister.SavingDocument(askForName);
            }

            public virtual void SaveDocument()
            {
                if (!DocumentExists)
                    throw new ApplicationException("!DocumentExists");
                Persister.SaveDocument(Document);
                Document.Name = Persister.LocalName;
                Document.OnSaveDocument();
            }

            public virtual bool CanCloseNotSavedDocument()
            {
                return true;
            }

            public virtual void CloseDocument()
            {
                if (!DocumentExists)
                    throw new ApplicationException("!DocumentExists");
                Document.OnCloseDocument();
                Document = null;
            }

            protected virtual string DefaultName
            {
                get { return "Document"; }
            }

            protected abstract DocumentBase CreateDocument(string name);
        }
    }
}
