﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using MiniLib.Model;

namespace MiniLib.Persistence
{
    public abstract class PersisterBase
    {
        private FileInfo _fileInfo = null;

        protected PersisterBase()
        {
        }

        public FileInfo FileInfo
        {
            get { return _fileInfo; }
            protected set { _fileInfo = value; }
        }

        public string FullFileName
        {
            get
            {
                return (_fileInfo != null ? _fileInfo.FullName : null);
            }
            protected set
            {
                if (String.IsNullOrEmpty(value))
                    _fileInfo = null;
                else
                    _fileInfo = new FileInfo(value);
            }
        }

        public string LocalName
        {
            get
            {
                if (_fileInfo == null)
                    throw new ApplicationException("_fileInfo == null");
                return Path.GetFileNameWithoutExtension(_fileInfo.FullName);
            }
        }

        public virtual bool OpeningDocument()
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                SetOpenFileDialog(openFileDialog);
                switch (openFileDialog.ShowDialog())
                {
                    case DialogResult.OK:
                        FullFileName = openFileDialog.FileName;
                        break;
                    default:
                        return false;
                }
            }
            return true;
        }

        public virtual bool SavingDocument(bool askForName)
        {
            if (FullFileName == null || askForName)
            {
                using (SaveFileDialog saveFileDialog = new SaveFileDialog())
                {
                    SetSaveFileDialog(saveFileDialog);
                    switch (saveFileDialog.ShowDialog())
                    {
                        case DialogResult.OK:
                            FullFileName = saveFileDialog.FileName;
                            break;
                        default:
                            return false;
                    }
                }
            }
            return true;
        }

        public abstract void LoadDocument(DocumentBase Document);
        public abstract void SaveDocument(DocumentBase Document);

        protected virtual void SetOpenFileDialog(OpenFileDialog openFileDialog)
        {
            openFileDialog.Title = "Select a document file";
            openFileDialog.FileName = FullFileName;
            openFileDialog.AddExtension = true;
            openFileDialog.RestoreDirectory = true;
        }

        protected virtual void SetSaveFileDialog(SaveFileDialog saveFileDialog)
        {
            saveFileDialog.Title = "Select a document file";
            saveFileDialog.FileName = FullFileName;
            saveFileDialog.AddExtension = true;
            saveFileDialog.OverwritePrompt = true;
            saveFileDialog.RestoreDirectory = true;
        }
    }
}
