﻿using System;
using System.Windows.Forms;
using System.ComponentModel;

namespace MiniLib.Presentation
{
    public partial class Grid : UserControl
    {
        private readonly BindingSource _bindingSource = new BindingSource();

        public event EventHandler CurrentChanged;

        public Grid()
        {
            InitializeComponent();
            _dataGridView.DataSource = _bindingSource;
            _bindingSource.CurrentChanged += delegate(object sender, EventArgs e)
                { OnCurrentChanged(); };
        }

        [Category("MiniLib")]
        public string Caption
        {
            get { return _captionLabel.Text; }
            set { _captionLabel.Text = value; }
        }

        [Browsable(false)]
        public object DataSource
        {
            get { return _bindingSource.DataSource; }
            set { _bindingSource.DataSource = value; }
        }

        [Browsable(false)]
        public object Current
        {
            get { return _bindingSource.Current; }
        }

        public void AddItem(object item)
        {
            _bindingSource.Add(item);
        }

        public void RemoveItem(object item)
        {
            _bindingSource.Remove(item);
        }

        //  Reread all the items in the list and refresh their displayed values
        public void RefreshItems()
        {
            _bindingSource.ResetBindings(false);
        }

        //  Reread the specified item and refresh its displayed value
        public void RefreshItem(object item)
        {
            int index = _bindingSource.IndexOf(item);
            if (index != -1)
                _bindingSource.ResetItem(index);
        }

        protected virtual void OnCurrentChanged()
        {
            if (CurrentChanged != null)
                CurrentChanged(this, EventArgs.Empty); 
        }
    }
}
